@@ -67,8 +67,6 @@ USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <X11/extensions/Xv.h>
 #include "shadow.h"
 #include "i830.h"
-#include "i830_display.h"
-#include "i830_bios.h"
 #include "i830_video.h"
 #if HAVE_SYS_MMAN_H && HAVE_MPROTECT
 #include <sys/mman.h>
@@ -232,25 +230,20 @@ const OptionInfoRec *I830AvailableOptions(int chipid, int busid)
 
 static Bool I830GetRec(ScrnInfoPtr scrn)
 {
-	intel_screen_private *intel;
-
 	if (scrn->driverPrivate)
 		return TRUE;
-	intel = scrn->driverPrivate = xnfcalloc(sizeof(intel_screen_private), 1);
+	scrn->driverPrivate = xnfcalloc(sizeof(intel_screen_private), 1);
+
 	return TRUE;
 }
 
 static void I830FreeRec(ScrnInfoPtr scrn)
 {
-	intel_screen_private *intel;
-
 	if (!scrn)
 		return;
 	if (!scrn->driverPrivate)
 		return;
 
-	intel = intel_get_screen_private(scrn);
-
 	xfree(scrn->driverPrivate);
 	scrn->driverPrivate = NULL;
 }
@@ -403,71 +396,6 @@ int i830_pad_drawable_width(int width, int cpp)
 	return (width + 63) & ~63;
 }
 
-static Bool i830_xf86crtc_resize(ScrnInfoPtr scrn, int width, int height)
-{
-#ifdef DRI2
-	intel_screen_private *intel = intel_get_screen_private(scrn);
-	int old_width = scrn->displayWidth;
-#endif
-	int old_x = scrn->virtualX;
-	int old_y = scrn->virtualY;
-
-	if (old_x == width && old_y == height)
-		return TRUE;
-
-	scrn->virtualX = width;
-	scrn->virtualY = height;
-#ifdef DRI2
-	if (intel->front_buffer) {
-		i830_memory *new_front, *old_front;
-		Bool tiled;
-		ScreenPtr screen = screenInfo.screens[scrn->scrnIndex];
-
-		scrn->displayWidth = i830_pad_drawable_width(width, intel->cpp);
-		tiled = i830_tiled_width(intel, &scrn->displayWidth, intel->cpp);
-		xf86DrvMsg(scrn->scrnIndex, X_INFO,
-			   "Allocate new frame buffer %dx%d stride %d\n", width,
-			   height, scrn->displayWidth);
-		intel_sync(scrn);
-		i830WaitForVblank(scrn);
-		new_front = i830_allocate_framebuffer(scrn);
-		if (!new_front) {
-			scrn->virtualX = old_x;
-			scrn->virtualY = old_y;
-			scrn->displayWidth = old_width;
-			return FALSE;
-		}
-		old_front = intel->front_buffer;
-		intel->front_buffer = new_front;
-		i830_set_pixmap_bo(screen->GetScreenPixmap(screen),
-				   new_front->bo);
-		scrn->fbOffset = intel->front_buffer->offset;
-
-		screen->ModifyPixmapHeader(screen->GetScreenPixmap(screen),
-					   width, height, -1, -1,
-					   scrn->displayWidth * intel->cpp,
-					   NULL);
-
-		/* ick. xf86EnableDisableFBAccess smashes the screen pixmap devPrivate,
-		 * so update the value it uses
-		 */
-		scrn->pixmapPrivate.ptr = NULL;
-		xf86DrvMsg(scrn->scrnIndex, X_INFO,
-			   "New front buffer at 0x%lx\n",
-			   intel->front_buffer->offset);
-		i830_set_new_crtc_bo(scrn);
-		intel_sync(scrn);
-		i830WaitForVblank(scrn);
-		i830_free_memory(scrn, old_front);
-	}
-#endif
-	return TRUE;
-}
-
-static const xf86CrtcConfigFuncsRec i830_xf86crtc_config_funcs = {
-	i830_xf86crtc_resize
-};
-
 /*
  * DRM mode setting Linux only at this point... later on we could
  * add a wrapper here.
@@ -1173,7 +1101,9 @@ I830ScreenInit(int scrnIndex, ScreenPtr screen, int argc, char **argv)
 	ScrnInfoPtr scrn = xf86Screens[screen->myNum];
 	intel_screen_private *intel = intel_get_screen_private(scrn);
 	VisualPtr visual;
+#ifdef INTEL_XVMC
 	MessageType from;
+#endif
 	struct pci_device *const device = intel->PciInfo;
 	int fb_bar = IS_I9XX(intel) ? 2 : 0;
 
@@ -1192,35 +1122,18 @@ I830ScreenInit(int scrnIndex, ScreenPtr screen, int argc, char **argv)
 	 * actual memory allocation, so alignment and things will cause less than
 	 * VideoRam to be actually used.
 	 */
-	if (intel->pEnt->device->videoRam == 0) {
-		from = X_DEFAULT;
-		scrn->videoRam = intel->FbMapSize / KB(1);
-	} else {
-#if 0
-		from = X_CONFIG;
-		scrn->videoRam = intel->pEnt->device->videoRam;
-#else
-		/* Disable VideoRam configuration, at least for now.  Previously,
-		 * VideoRam was necessary to avoid overly low limits on allocated
-		 * memory, so users created larger, yet still small, fixed allocation
-		 * limits in their config files.  Now, the driver wants to allocate more,
-		 * and the old intention of the VideoRam lines that had been entered is
-		 * obsolete.
-		 */
-		from = X_DEFAULT;
-		scrn->videoRam = intel->FbMapSize / KB(1);
-
+	scrn->videoRam = intel->FbMapSize / KB(1);
+	if (intel->pEnt->device->videoRam != 0) {
 		if (scrn->videoRam != intel->pEnt->device->videoRam) {
 			xf86DrvMsg(scrn->scrnIndex, X_WARNING,
-				   "VideoRam configuration found, which is no longer "
-				   "recommended.\n");
+				   "VideoRam configuration found, which is no "
+				   "longer used.\n");
 			xf86DrvMsg(scrn->scrnIndex, X_INFO,
-				   "Continuing with default %dkB VideoRam instead of %d "
-				   "kB.\n",
+				   "Continuing with (ignored) %dkB VideoRam "
+				   "instead of %d kB.\n",
 				   scrn->videoRam,
 				   intel->pEnt->device->videoRam);
 		}
-#endif
 	}
 
 	scrn->videoRam = device->regions[fb_bar].size / 1024;
