@@ -326,7 +326,6 @@ static void i830_texture_setup(PicturePtr picture, PixmapPtr pixmap, int unit)
 			  (FILTER_LINEAR << TM0S3_MIN_FILTER_SHIFT));
 		break;
 	default:
-		filter = 0;
 		FatalError("Bad filter 0x%x\n", picture->filter);
 	}
 	filter |= (MIPFILTER_NONE << TM0S3_MIP_FILTER_SHIFT);
@@ -555,7 +554,7 @@ i830_prepare_composite(int op, PicturePtr source_picture,
 
 	if(i830_uxa_pixmap_is_dirty(source) ||
 	   (mask && i830_uxa_pixmap_is_dirty(mask)))
-		intel_batch_pipelined_flush(scrn);
+		intel_batch_emit_flush(scrn);
 
 	intel->needs_render_state_emit = TRUE;
 
