@@ -38,6 +38,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "i915_drm.h"
 #include <string.h>
 #include <sys/mman.h>
+#include <errno.h>
 
 const int I830CopyROP[16] = {
 	ROP_0,			/* GXclear */
@@ -92,7 +93,7 @@ i830_get_aperture_space(ScrnInfoPtr scrn, drm_intel_bo ** bo_table,
 
 	bo_table[0] = intel->batch_bo;
 	if (drm_intel_bufmgr_check_aperture_space(bo_table, num_bos) != 0) {
-		intel_batch_flush(scrn, FALSE);
+		intel_batch_submit(scrn);
 		bo_table[0] = intel->batch_bo;
 		if (drm_intel_bufmgr_check_aperture_space(bo_table, num_bos) !=
 		    0) {
@@ -155,7 +156,7 @@ i830_uxa_pixmap_compute_size(PixmapPtr pixmap,
 		 */
 		size = *stride * ALIGN(h, 2);
 	} else {
-		int aligned_h = h;
+		int aligned_h;
 		if (*tiling == I915_TILING_X)
 			aligned_h = ALIGN(h, 8);
 		else
@@ -188,12 +189,17 @@ i830_uxa_prepare_solid(PixmapPtr pixmap, int alu, Pixel planemask, Pixel fg)
 {
 	ScrnInfoPtr scrn = xf86Screens[pixmap->drawable.pScreen->myNum];
 	intel_screen_private *intel = intel_get_screen_private(scrn);
-	unsigned long pitch;
 	drm_intel_bo *bo_table[] = {
 		NULL,		/* batch_bo */
 		i830_get_pixmap_bo(pixmap),
 	};
 
+	if (IS_GEN6(intel)) {
+		intel_debug_fallback(scrn,
+				     "Sandybridge BLT engine not supported\n");
+		return FALSE;
+	}
+
 	if (!UXA_PM_IS_SOLID(&pixmap->drawable, planemask)) {
 		intel_debug_fallback(scrn, "planemask is not solid\n");
 		return FALSE;
@@ -212,8 +218,6 @@ i830_uxa_prepare_solid(PixmapPtr pixmap, int alu, Pixel planemask, Pixel fg)
 	if (!intel_check_pitch_2d(pixmap))
 		return FALSE;
 
-	pitch = i830_pixmap_pitch(pixmap);
-
 	if (!i830_pixmap_pitch_is_aligned(pixmap)) {
 		intel_debug_fallback(scrn, "pixmap pitch not aligned");
 		return FALSE;
@@ -298,6 +302,12 @@ i830_uxa_prepare_copy(PixmapPtr source, PixmapPtr dest, int xdir,
 		i830_get_pixmap_bo(dest),
 	};
 
+	if (IS_GEN6(intel)) {
+		intel_debug_fallback(scrn,
+				     "Sandybridge BLT engine not supported\n");
+		return FALSE;
+	}
+
 	if (!UXA_PM_IS_SOLID(&source->drawable, planemask)) {
 		intel_debug_fallback(scrn, "planemask is not solid");
 		return FALSE;
@@ -568,69 +578,53 @@ static Bool i830_uxa_prepare_access(PixmapPtr pixmap, uxa_access_t access)
 	intel_screen_private *intel = intel_get_screen_private(scrn);
 	struct intel_pixmap *priv = i830_get_pixmap_intel(pixmap);
 	dri_bo *bo = priv->bo;
+	int ret;
 
 	if (!list_is_empty(&priv->batch) &&
 	    (access == UXA_ACCESS_RW || priv->batch_write_domain))
-		intel_batch_flush(scrn, TRUE);
+		intel_batch_submit(scrn);
 
-	/* No VT sema or GEM?  No GTT mapping. */
-	if (!scrn->vtSema) {
-		if (dri_bo_map(bo, access == UXA_ACCESS_RW) != 0)
-			return FALSE;
-	} else if (bo->size < intel->max_gtt_map_size) {
-		if (drm_intel_gem_bo_map_gtt(bo)) {
+	if (bo->size > intel->max_gtt_map_size) {
+		ret = dri_bo_map(bo, access == UXA_ACCESS_RW);
+		if (ret != 0) {
 			xf86DrvMsg(scrn->scrnIndex, X_WARNING,
-				   "%s: bo map failed\n", __FUNCTION__);
+				   "%s: bo map failed: %s\n",
+				   __FUNCTION__,
+				   strerror(-ret));
 			return FALSE;
 		}
 	} else {
-		if (dri_bo_map(bo, access == UXA_ACCESS_RW) != 0) {
+		ret = drm_intel_gem_bo_map_gtt(bo);
+		if (ret != 0) {
 			xf86DrvMsg(scrn->scrnIndex, X_WARNING,
-				   "%s: bo map failed\n", __FUNCTION__);
+				   "%s: gtt bo map failed: %s\n",
+				   __FUNCTION__,
+				   strerror(-ret));
 			return FALSE;
 		}
 	}
+
 	pixmap->devPrivate.ptr = bo->virtual;
 
-	/* This acts as a synchronisation point. */
-	while (!list_is_empty(&intel->flush_pixmaps)) {
-		struct intel_pixmap *entry;
-
-		entry = list_first_entry(&intel->flush_pixmaps,
-					 struct intel_pixmap,
-					 flush);
-
-		entry->flush_read_domains = entry->flush_write_domain = 0;
-		list_del(&entry->flush);
-	}
-
 	return TRUE;
 }
 
 static void i830_uxa_finish_access(PixmapPtr pixmap)
 {
 	dri_bo *bo = i830_get_pixmap_bo(pixmap);
+	ScreenPtr screen = pixmap->drawable.pScreen;
+	ScrnInfoPtr scrn = xf86Screens[screen->myNum];
+	intel_screen_private *intel = intel_get_screen_private(scrn);
 
-	if (bo) {
-		ScreenPtr screen = pixmap->drawable.pScreen;
-		ScrnInfoPtr scrn = xf86Screens[screen->myNum];
-		intel_screen_private *intel = intel_get_screen_private(scrn);
+	if (bo == intel->front_buffer->bo)
+		intel->need_flush = TRUE;
 
-		if (bo == intel->front_buffer->bo)
-			intel->need_flush = TRUE;
+	if (bo->size > intel->max_gtt_map_size)
+		dri_bo_unmap(bo);
+	else
+		drm_intel_gem_bo_unmap_gtt(bo);
 
-		if (!scrn->vtSema) {
-			dri_bo_unmap(bo);
-			pixmap->devPrivate.ptr = NULL;
-			return;
-		}
-
-		if (bo->size < intel->max_gtt_map_size)
-			drm_intel_gem_bo_unmap_gtt(bo);
-		else
-			dri_bo_unmap(bo);
-		pixmap->devPrivate.ptr = NULL;
-	}
+	pixmap->devPrivate.ptr = NULL;
 }
 
 static Bool
